#ifndef _SW_RATE_H
#define _SW_RATE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <mysql.h>

struct _swrate;
typedef struct _swrate SWRATE;

typedef enum
{
	SWR_INT_WEEK = 0,
	SWR_INT_MONTH,
	SWR_INT_DAY,
	SWR_INT_HOUR,
	SWR_INT_MINUTE,
	SWR_INT_LAST,
} swr_intunit_t;

typedef enum
{
	SWR_STM_CREATE = 0,
	SWR_STM_1ST,
	SWR_STM_TOPUP,
	SWR_STM_1ST_TZ,
	SWR_STM_CREATE_TZ,
	SWR_STM_LAST,
} swr_startmode_t;

typedef enum
{
	SWR_COM_CARRY_OVER = 0,
	SWR_COM_CARRY_OVER_TIMELY,
	SWR_COM_DISCARD,
	SWR_COM_LAST,
} swr_carryovermode_t;

typedef enum
{
	SWR_CTYPE_CALL = 0,
	SWR_CTYPE_SMS
	
} swr_calltype_t;

typedef struct
{
	unsigned int contract_id;
	unsigned int profile_id;
	double interval_charge;
	int interval_free_time;
	double interval_free_cash;
	swr_intunit_t interval_unit;
	unsigned int interval_count;
	unsigned int product_id; // unused
	int prepaid:1;

	// fields below are valid only when this was allocated by swrate_profile_new
	char prepaid_library[16];
} swr_profile_t;

typedef struct
{
	unsigned int fee_id;
	char pattern[256];
	double on_init_rate;
	int on_init_interval;
	double on_follow_rate;
	int on_follow_interval;
	double off_init_rate;
	int off_init_interval;
	double off_follow_rate;
	int off_follow_interval;
	unsigned int zone_id;
	int use_free_time:1;
} swr_fee_t;

typedef struct
{
	int init_interval;
	double init_rate;
	int follow_interval;
	double follow_rate;
	unsigned int validity;
	int use_free_time:1;
} swr_rate_t;

typedef struct
{
	time_t start; /* seconds since 1970-01-01 00:00:00 */
	time_t end; /* dito */
} swr_offpeak_t;

typedef struct
{
	unsigned int balance_id;
	double cash_balance;
	double cash_balance_interval;
	int free_time_balance;
	int free_time_balance_interval;
	time_t start_time;
	time_t end_time;
} swr_cbalance_t;

typedef struct
{
	unsigned int package_id;
	double underrun_profile_threshold;
	double underrun_lock_threshold;
	int underrun_lock_level;
	time_t underrun_lock_applied;
	time_t underrun_profiles_applied;
	unsigned int underrun_profiles_count;
} swr_package_info_t;

typedef struct
{
	unsigned int balance_id;
	int duration;
} swr_prepaid_t;

typedef struct
{
	unsigned int contract_id;
	time_t create_timestamp;
	time_t modify_timestamp;
	unsigned int reseller_id;
	unsigned int package_id;
	swr_intunit_t interval_unit;
	unsigned int interval_value;
	swr_startmode_t start_mode;
	swr_carryovermode_t carry_over_mode;
	unsigned int notopup_discard_intervals;
	double underrun_profile_threshold;
	double underrun_lock_threshold;
	int underrun_lock_level;
	unsigned int underrun_profiles_count;
	char product_class[64];
} swr_contract_t;



#pragma GCC visibility push(default)


/*
 * Initializes library and returns handle used for subsequent
 * function calls. Must be called before any other function.
 * Returns -1 on error.
 * DEPRECATED! Use swrate_new() instead.
 */
int swrate_init(SWRATE *handle, const char* host, int port,
	const char* db, const char* user, const char* pass,
	const char *accounting_db,
	int log_debug, int log_to_syslog, int log_to_stdout);

SWRATE *swrate_new(const char* host, int port,
	const char* db, const char* user, const char* pass,
	const char *accounting_db,
	int log_debug, int log_to_syslog, int log_to_stdout);

/*
 * Check if swrate_new() performed successfully.
 * Returns 1 if OK, 0 otherwise.
 */
int swrate_new_ok(SWRATE *handle);

/*
 * Cleans up internal library data structures. Must be called
 * at very last function.
 * DEPRECATED! Use swrate_free() instead.
 */
int swrate_cleanup(SWRATE *handle);

int swrate_free(SWRATE *handle);

/*
 * Returns the last error string from swrate
 */
const char *swrate_error(SWRATE *handle);

/*
 * Configures a separate MySQL connection for writing
 * accounting (prepaid) data records.
 * The database name is already known from swrate_new()
 * or _init() (accounting_db).
 */
int swrate_accounting_db(SWRATE *handle, const char* host, int port,
	const char* user, const char* pass);

/*
 * Configures the creds for the provisioning DB
 */
int swrate_provisioning_db(SWRATE *handle, const char* host, int port,
	const char* user, const char* pass, const char *db);

/*
 * Allocate a new swr_profile_t struct. Must be free'd using swrate_profile_free()
 */
swr_profile_t *swrate_profile_new(void);

/*
 * Free struct allocated by swrate_profile_new()
 */
void swrate_profile_free(swr_profile_t *);

/*
 * If call_duration == -1:
 * Calculates maximum duration in seconds for user <uuid>
 * for originating or terminating direction <direction=in|out>
 * to destination <destination> at start time <start_time>.
 * If call_duration >= 0:
 * Subtracts the prepaid call from the user's balance.
 *
 * Result of calculation is returned in 'p', which can be
 * NULL.
 *
 * Returns -1 on error, 0 on success
 */
int swrate_prepaid_domain(swr_prepaid_t *p, SWRATE *h, const char *uid, const char *dst_num,
		const char *domain, time_t start_time, int call_duration, unsigned int free_time_used,
		const char *callid);
int swrate_prepaid_hr_domain(swr_prepaid_t *p, SWRATE *h, const char *uid, const char *dst_num,
		const char *domain, struct timeval start_time, double call_duration,
		unsigned int free_time_used, const char *callid);
int swrate_prepaid_hr_dir_ab_domain(swr_prepaid_t *p, SWRATE *h, const char *direction,
		const char *uid, const char *src_num, const char *src_domain, const char *dst_num,
		const char *dst_domain, struct timeval start_time, double call_duration,
		unsigned int free_time_used, const char *callid);
int swrate_prepaid_hr_dir_ab_domain_srcip(swr_prepaid_t *p, SWRATE *h, const char *direction,
		const char *uid, const char *src_num, const char *src_domain, const char *src_ip,
		const char *dst_num, const char *dst_domain, struct timeval start_time,
		double call_duration, unsigned int free_time_used, const char *callid);
int swrate_prepaid_hr_dir_ab_domain_srcip_uids(swr_prepaid_t *p, SWRATE *h, const char *direction,
		const char *uid, const char *src_num, const char *src_domain, const char *src_ip,
		const char *src_uid,
		const char *dst_num, const char *dst_domain, const char *dst_uid,
		struct timeval start_time,
		double call_duration, unsigned int free_time_used, const char *callid);

int swrate_prepaid(swr_prepaid_t *p, SWRATE *h, const char *uid, const char *dst_num,
		time_t start_time, int call_duration, unsigned int free_time_used, const char *callid);
int swrate_prepaid_hr(swr_prepaid_t *p, SWRATE *h, const char *uid, const char *dst_num,
		struct timeval start_time, double call_duration, unsigned int free_time_used,
		const char *callid);


int swrate_get_rate_dir_ab_domain_srcip(swr_rate_t *r, SWRATE *h, const char *direction,
		const char *uid, const char *src_num, const char *src_domain, const char *src_ip,
		const char *dst_num, const char *dst_domain, time_t start_time);
int swrate_get_rate_dir_ab_domain(swr_rate_t *r, SWRATE *h, const char *direction, const char *uid,
		const char *src_num, const char *src_domain, const char *dst_num,
		const char *dst_domain, time_t start_time);
int swrate_get_rate_domain(swr_rate_t *r, SWRATE *h, const char *uid, const char *dst_num,
		const char *domain, time_t start_time);
int swrate_get_rate(swr_rate_t *r, SWRATE *h, const char *uid, const char *dst_num, time_t start_time);
int swrate_get_peer_ab_rate_srcip(swr_rate_t *r, SWRATE *h, unsigned int peer_id,
		const char *src_num, const char *src_domain, const char *src_ip,
		const char *dst_num, const char *dst_domain, time_t start_time);
int swrate_get_peer_ab_rate(swr_rate_t *r, SWRATE *h, unsigned int peer_id, const char *src_num,
		const char *src_domain, const char *dst_num, const char *dst_domain, time_t start_time);
int swrate_get_peer_rate(swr_rate_t *r, SWRATE *h, unsigned int peer_id, const char *dst_num,
		const char *domain, time_t start_time);
int swrate_get_current_balance(swr_cbalance_t *b, SWRATE *h, const char *uid);
int swrate_get_current_contract_balance(swr_cbalance_t *b, SWRATE *h, unsigned int);
unsigned int swrate_get_subscriber_account_id(SWRATE *h, const char *uid);
int swrate_transfer_balance(SWRATE *h, unsigned int destination, unsigned int source);


int swrate_get_subscriber_billing_profile(swr_profile_t *p, SWRATE *h, time_t start,
		const char *uid, const char *ip);



#pragma GCC visibility pop



#ifdef __cplusplus
}
#endif

#endif /* _SW_RATE_H */
